#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__autho__ = "Tony.Don"
__lastupdatetime__ = "2017/09/29"

import sqlite3
'''
class \r\n
    core.dataconfig.db(object)\r\n
description\r\n
    数据库帮助类(SQLLITE)\r\n
attribute\r\n
    private __database__\r\n
        type:string\r\n
        description:数据库\r\n
    private __conn__\r\n
        type:sqlconn\r\n
        description:连接对象\r\n
    private __cursor__\r\n
        type:sqlcursor\r\n
        description:cursor对象\r\n
'''
class base(object):
    __database__=""
    __conn__=None
    __cursor__=None
    def __init__(self,database):
        self.__database__=database
        self.__conn__=sqlite3.connect(database)
        self.__cursor__=self.__conn__.cursor()
        
    def __del__(self):
        self.__database__=None
        self.__conn__=None
        self.__cursor__=None
    
    def __getattribute__(self,name):
        if name == "__conn__":
            result=object.__getattribute__(self,name)
            return result if result else 0
        elif name == "__cursor__":
            result=object.__getattribute__(self,name)
            return result if result else 0
        else:
            return object.__getattribute__(self,name)


class ApiConfig():
    '''Api Config
        
        Args:
            key (str): api keyword.
            uri (str): api uri.
            apiType (str): api Type, default value is "get".
            argsTemplete (str): api param format(Example "param1={0}&param2={1}"), default is "param1={0}".
            descrition (str): api descrition.
    '''
    def __init__(self, key:str, uri:str, apiType:str="get", argsTemplete:str="param1={0}", descrition:str=""):
        self._key=key
        self._uri=uri
        self._type=apiType
        self._argsTemplete=argsTemplete
        self._descrition=descrition

    @property
    def Key(self):
        return self._key

    @property
    def Uri(self):
        return self._uri

    @property
    def Type(self):
        return self._type

    @property
    def ArgsTemplete(self):
        return self._argsTemplete

    @property
    def Descrition(self):
        return self._descrition

class Config():
    '''SMS CONFIG
       
       Args:
            platforName (str): open api name.
            account (str): open api account.
            secret (str): open api secret.
            certificate (str): open api private certificate.
            version (str): open api version, default value is "1.0"

       Property:
            PlatformName (str): get the open api platform name.
            Account (str): get open api platform account.
            Secret (str): get open api platform secret.
            Certificate (str): (https) get open api plate form certificate.
            Apis (class<ApiList>): get apilist.
            Version (str): get open api version, default value is "1.0"
    '''
    
    def __init__(self,platformName:str, account:str, secret:str, certificate:str, version:str="1.0"):
        self._platformName=platformName
        self._account=account
        self._secret=secret
        self._certificate=certificate
        self._apiList=[]
        self._version=version

    def AddApi(self, apiConfig:ApiConfig):
        '''AddApi

            Args:
                apiConfig (Class<ApiConfig>):api config
        '''
        self._apiList.append(apiConfig)
    @property
    def Version(self):
        return self._version

    @property
    def PlatformName(self):
        return self._platformName
    
    @property
    def Account(self):
        return self._account

    @property
    def Secret(self):
        return self._secret

    @property
    def Certificate(self):
        return self._certificate

    @property
    def Apis(self):
        return self._apiList





import os
from TDhelper.db.Db.base import base

m_dbDao=None

def dbInstance(dbPath,dbName):
        if not m_dbDao:
                dbPath=str(os.path.split(os.path.realpath(__file__))[0])+"\\"+dbName
                m_dbDao=base(dbPath)


from abc import ABCMeta, abstractmethod
import threading
import time


class absDevice(threading.Thread):
    def __init__(self, deviceName: str, revolveScope=(0, 180), reset=0.0):
        threading.Thread.__init__(self)
        #  self.id= id
        self.name = deviceName  # 设备名称.
        self.revolveScope = revolveScope  # 可转动范围限制.
        self.currentValue = -1  # 当前设备状态值.
        self.resetValue = reset  # 复位参数.
        self.ctrStatus = False  # 控制状态
        self.deviceStatus = False  # 设备状态
        self.start()

    def run(self):
        count = 0
        while True:
            if count == 0:
                count += 1
                self.deviceStatus= True
                print("%s 动力设备准备就绪." % self.name)
            if self.ctrStatus:
                self.notify()
            self.ctrStatus = False
            time.sleep(0.1)

    def reset(self):
        self.ctrl(self.resetValue)

    @abstractmethod
    def notify(self):
        '''
            通知硬件设备控制.
        '''
        pass

    @abstractmethod
    def ctrl(self, scope: float = 0.0):
        '''
        控制接口
        - params
        -   scope: <float>, 控制舵机旋转到多少度.
        '''
        pass



'''
can insert console_scripts module by setup.py; after module install success,can use shell command.
e.g. 'console_scripts': [
            #'foo = demo:test',
            #'bar = demo:test',
            'saas = TDhelper.shellScripts.saasHelper:CMD'
        ],

e.g. shell command: saas --name [name] --path [path] --git [git url]
'''


from enum import Enum
import functools
import threading
import TDhelper.bin.globalvar

class Event:
    '''
        Initialization event class,and set this class state flag,if has EventList Variable state is True,else state is False
    '''
    def __init__(self):
        self._state=True
        self._currentEvent= None
        if TDhelper.bin.globalvar.getGlobalVariable("EventList")==None: #Check eventList object is define
            self._state=False

    def _keyconvert(self,key):
        if isinstance(key,str):
            return (str(self.__hash__())+key).lower()
        elif isinstance(key,Enum):
            return (str(self.__hash__())+key.value).lower()
        else:
            return None
    '''
        Check event key haven't set
        Return:
        if event key haven't set return True,else it return False
    '''
    def _checkKeyNotInEventList(self,key):
        return self._keyconvert(key) not in TDhelper.bin.globalvar.getGlobalVariable("EventList")

    def _getEventListObject(self,key):
        return TDhelper.bin.globalvar.getGlobalVariable("EventList")[self._keyconvert(key)]

    def getEventList(self):
        return TDhelper.bin.globalvar.getGlobalVariable("EventList")

    '''
        Register an event
        Parameter
            key:
                event index key
            func:
                if sys has happend an event,then callback func
            sync:
                default value is True so callback by synchronous,if you set value is False then call back by asynchronous
    '''
    def registerEvent(self,key,func,sync=True):
        if self._state:
            if self._checkKeyNotInEventList(self._keyconvert(key)):
                TDhelper.bin.globalvar.getGlobalVariable("EventList")[self._keyconvert(key)]={"func":func,"sync":sync}


    '''
        Delete an event in the Event List;if you delete then this event has happend we don't callback
    '''
    def delEvent(self,key):
        if self._state:
            if not self._checkKeyNotInEventList(self._keyconvert(key)):
                del TDhelper.bin.globalvar.getGlobalVariable("EventList")[self._keyconvert(key)]

    '''
        Callback
    '''
    def on(self,key,*args,**kw):
        try:
            if key:
                if self._state:
                    if self._keyconvert(key) in TDhelper.bin.globalvar.getGlobalVariable("EventList"):
                        self._currentEvent= key
                        if self._getEventListObject(key)["sync"]:
                            return self._getEventListObject(key)["func"](*args,**kw)
                        else:
                            #todo asynchronous call back
                            pass#threading.Thread(target=self._getEventListObject(key)["func"],args=[*args,**kw,]).start()
        except Exception as e:
            raise e
    def EVENT(self):
        return self._currentEvent

def keyconvert(key):
    if isinstance(key,str):
        return key.lower()
    elif isinstance(key,Enum):
        return key.value.lower()

def call(key,*args,**kw):
    try:
        key=keyconvert(key)
        if len(args)>0:
            if key in TDhelper.bin.globalvar.getGlobalVariable("EventList"):
                if TDhelper.bin.globalvar.getGlobalVariable("EventList")[key]["sync"]:
                    return TDhelper.bin.globalvar.getGlobalVariable("EventList")[key]["func"](*args,**kw)
                else:
                    pass#threading.Thread(target=bin.globalvar.getGlobalVariable("EventList")[key]["func"],args=[*args,**kw,]).start()
    except Exception as e:
        raise e

def trigger(EventType):
        def decorator(func):
            @functools.wraps(func)
            def expand(*args,**kw):
                #to add event start event
                args[0].on("begin"+str(EventType),*args,**kw)
                #to add event start event
                result=func(*args,**kw)
                #to add event complate event
                args[0].on("on"+str(EventType)+"complete",*args,**kw)
                #to add event complate event
                return result
            return expand
        return decorator    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pymongo
'''
class\r\n
    lib.db.mongodb.mongodbclient\r\n
description\r\n
    Mongodb database helper class\r\n
'''
class mongodbclient:
    client=None
    db=None
    def __init__(self):
        self.collection= None
       
    def loadCfg(self, **kwargs):
        '''
        加载MONGODB配置文件
        - parameters:
            kwargs: {url=mongodb://username:password@localhost:port, db= db_name}
        '''
        if not self.client or not self.db:
            if "url" in kwargs and 'db' in kwargs:
                if kwargs['url']:
                    try:
                        self.client=pymongo.MongoClient(kwargs['url'])
                        if self.client:
                            if kwargs['db']:
                                self.db=self.client[kwargs['db']]
                    except Exception as e:
                        raise e
                else:
                    raise Exception('mongodb url is none.')
            else:
                raise Exception("mongdb setting url or db is not define.")